# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
# sys.path.insert(0, os.path.abspath('..'))
sys.path.append('.')

# -- Dataset table listing -----------------------------------------------------
import pyterrier as pt
import textwrap

from extras import generate_includes
from extras import generate_extensions
generate_includes.setup()
if not "QUICK" in os.environ:
    generate_includes.experiment_includes()
generate_includes.dataset_include()
generate_includes.artifact_list_include()
generate_extensions.generate_extensions()

# -- Project information -----------------------------------------------------
import datetime
now = datetime.datetime.now()

project = 'PyTerrier'
copyright = '%d, the University of Glasgow' % (now.year)
author = 'Contributors to PyTerrier'

# The short X.Y version
version = ''
# The full version, including alpha/beta/rc tags
release = ''

html_title = f'{project} {release}'
html_short_title = project


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'extras.cite',
    'extras.schematic',
    'sphinx.ext.autodoc',
    'sphinx_autodoc_typehints',
    'sphinx.ext.coverage',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages',
    'sphinx.ext.napoleon',
    'sphinx_tabs.tabs',
    'enum_tools.autoenum',
    'extras.related',
    'extras.codeblock_footnotes',
    'sphinx_togglebutton',
    'sphinx_reredirects',
    "sphinx_design",
    'extras.how_to',
]


autodoc_typehints = "description"

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

autoclass_content = 'both'
always_use_bars_union = True

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path .
exclude_patterns = ['README.md', '_build', '_includes', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'furo'

# Avoid showing over-qualified name in the sidebar, reducing redundancy and avoiding wrapping in many cases.
# I.e., instead of:
#   MyClass
#     MyClass.my_method()
# It will show:
#    MyClass
#      my_method()
toc_object_entries_show_parents = 'hide'

# increasing sphinx width
# https://stackoverflow.com/a/43186995
def setup(app):
    app.add_css_file('my_theme.css')

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}

html_logo = "_static/pyterrier logo 478w.png"


autodoc_default_options = {
    'member-order': 'bysource',
}

autodoc_inherit_docstrings = False

# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'Pyterrierdoc'


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'Pyterrier.tex', 'PyTerrier Documentation',
     'Contributors to PyTerrier', 'manual'),
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'pyterrier', 'PyTerrier Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'PyTerrier', 'PyTerrier Documentation',
     author, 'PyTerrier', 'One line description of project.',
     'Miscellaneous'),
]


# -- Extension configuration -------------------------------------------------
# for Sphinx-1.4 or newer

extensions += ["myst_parser"]
source_suffix = ['.rst', '.md']

redirects = {
    "terrier-indexing": "terrier/indexing.html",
    "terrier-retrieval": "terrier/retrieval.html",
    "rewrite": "terrier/rewrite.html",
    "terrier-index-api": "terrier/how-to.html", # Most of the content moved here
}

linkcheck_anchors = False
linkcheck_timeout = 10
linkcheck_ignore = [r'https://.*acm.org/.*', r'https://www.java.com/.*'] # these domains block linkcheck



"""
Sphinx extension for creating how-to sections with automatic indexing.

This extension provides a `how-to` directive that formats how-to entries
and automatically generates an index of all how-tos in the documentation.
"""
import uuid
from docutils import nodes
from docutils.parsers.rst import directives
from docutils.statemachine import ViewList
from sphinx.util.docutils import SphinxDirective
from sphinx.domains.std import StandardDomain
from sphinx.util import logging

logger = logging.getLogger(__name__)


class HowToDirective(SphinxDirective):
    """
    Directive for creating how-to sections.
    
    Usage:
        .. how-to:: Title of the how-to
           
           Content goes here (parsed as RST)
    """
    
    has_content = True
    required_arguments = 1  # The title
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}
    
    def run(self):
        env = self.env
        
        # Initialize howto_list if it doesn't exist
        if not hasattr(env, 'howto_list'):
            env.howto_list = {}
        
        # Get the title from arguments
        title = ' '.join(self.arguments)
        label = uuid.uuid4().hex
        
        if env.docname not in env.howto_list:
            # Get the actual page title from the document metadata
            titles = []
            node = self.state.parent
            while node:
                if isinstance(node, nodes.section):
                    # Get section ID
                    section_id = (node.get('ids') or [None])[0]
                    
                    # Get section title
                    title_node = node.next_node(nodes.title)
                    section_title = title_node.astext() if title_node else 'Unknown Section'
                    titles.append(section_title)
                node = node.parent
            
            env.howto_list[env.docname] = {
                'docname': env.docname,
                'title': titles[-1] if len(titles) > 0 else env.docname,
                'how_tos': [],
            }
        env.howto_list[env.docname]['how_tos'].append({
            'title': title,
            'label': label,
        })
        
        # Create the output nodes
        result_nodes = []
        
        # Add horizontal rule
        hr_node = nodes.transition()
        result_nodes.append(hr_node)
        
        target_node = nodes.target('', '', ids=[label], names=[label])
        self.state.document.note_explicit_target(target_node)
        result_nodes.append(target_node)

        # Add title as a section
        section_node = nodes.section()
        section_node['classes'].append('howto-section')
        
        # Create title node with proper formatting
        title_nodes, messages = self.state.inline_text(title, self.lineno)
        title_node = nodes.title(title, '', *title_nodes)
        section_node += title_node
        
        self.state.nested_parse(self.content, self.content_offset, section_node)
        
        result_nodes.append(section_node)
        
        return result_nodes


def init_howto_list(app, env):
    """Initialize the howto_list if it doesn't exist."""
    if not hasattr(env, 'howto_list'):
        env.howto_list = {}
    return []


def merge_howto_list(app, env, docnames, other):
    """Merge howto_list from parallel builds."""
    if not hasattr(env, 'howto_list'):
        env.howto_list = {}
    if hasattr(other, 'howto_list'):
        env.howto_list.update(other.howto_list)


def purge_howtos(app, env, docname):
    """Remove how-tos for a document that's being rebuilt."""
    if hasattr(env, 'howto_list') and docname in env.howto_list:
        del env.howto_list[docname]


def collect_howtos(app):
    """Collect all how-tos and generate the index page."""
    env = app.env
    
    if app.builder.embedded or not hasattr(env, 'howto_list') or len(env.howto_list) == 0:
        # Building embedded (e.g. htmlhelp or ePub) or there were no howtos found
        return []

    pages = []
    for page in sorted(env.howto_list.values(), key=lambda x: x['title']):
        # Generate URLs for each how-to
        for howto in sorted(page['how_tos'], key=lambda x: x['title']):
            # Use the document name to get the URI, then append the anchor
            howto['url'] = app.builder.get_target_uri(page['docname']) + '#' + howto['label']
        pages.append(page)

    return [('how-tos', {'pages': pages}, 'how-tos.html')]


def setup(app):
    """Setup function for the Sphinx extension."""
    app.add_directive('how-to', HowToDirective)
    
    # Connect to proper lifecycle events for persistence
    app.connect('env-get-updated', init_howto_list)
    app.connect('env-purge-doc', purge_howtos)
    app.connect('env-merge-info', merge_howto_list)
    app.connect('html-collect-pages', collect_howtos)
    
    StandardDomain._virtual_doc_names['how-tos'] = ('how-tos', "How-To Guides")
    
    return {
        'version': '0.1',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }
